package com.eco.plugin.xx.hlzjsso.filter;

import com.eco.plugin.xx.hlzjsso.config.PluginSimpleConfig;
import com.eco.plugin.xx.hlzjsso.utils.*;
import com.fr.decision.authority.data.User;
import com.fr.decision.fun.impl.AbstractGlobalRequestFilterProvider;
import com.fr.json.JSONArray;
import com.fr.json.JSONObject;
import com.fr.plugin.context.PluginContexts;
import com.fr.record.analyzer.EnableMetrics;
import com.fr.stable.fun.Authorize;

import javax.servlet.FilterChain;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

@EnableMetrics
@Authorize(callSignKey = "com.eco.plugin.xx.hlzjsso")
public class SSOFilter extends AbstractGlobalRequestFilterProvider {
    @Override
    public String filterName() {
        return "hlzjssoFilter";
    }

    @Override
    public String[] urlPatterns() {
        return new String[]{"/*"};
    }

    @Override
    public void doFilter(HttpServletRequest req, HttpServletResponse res, FilterChain chain ){

        if(PluginContexts.currentContext().isAvailable()){
            PluginSimpleConfig psc = PluginSimpleConfig.getInstance();

            //token为空,不做拦截
            String token = req.getParameter(psc.getParamname());
            String url = FRUtils.getAllUrl(req);

            String method = req.getMethod();
            //登陆事件
            if(url.endsWith("decision/login") && method.equals("POST")){
                login(req,res);
                return ;
            }

            if(Utils.isNullStr(token)){
                release(req,res,chain);
                return;
            }

            String decryptToken = null;
            try {
                decryptToken = EncryptUtils.rsaDecrypt(token,psc.getSecret());
            } catch (Exception e) {
                ResponseUtils.failedResponse(res,"token解密失败:"+e.getMessage());
                return ;
            }

            if(Utils.isNullStr(decryptToken)){
                ResponseUtils.failedResponse(res,"token解密失败!");
                return ;
            }

            JSONObject tokenJson = new JSONObject(decryptToken);
            String username = tokenJson.getString("username");
            Long timestamp = tokenJson.getLong("timestamp");
            Long now = System.currentTimeMillis();

            boolean isWhite = IPWhiteUtils.inWhite(req,psc.getWhite());

            //外网而且权限控制功能开关为开
            if(!isWhite && psc.getIsAll()){

                boolean isAdmin = false;
                try {
                    isAdmin = Utils.isAdmin(username);
                } catch (Exception e) {
                    ResponseUtils.failedResponse(res,"判断管理员失败!");
                    return ;
                }

                //外网环境且管理员控制为开，管理员不允许单点登陆
                if(isAdmin && psc.getIsAdmin()){
                    ResponseUtils.failedResponse(res,"外网环境管理员账户不允许单点登陆!");
                    return ;
                }

                //外网环境切账户转换为开,对普通用户进行转换
                //判断是否以A结尾，A结尾为外网账户，如果不是将A添加到用户名结尾
                boolean endwithA = username.endsWith(psc.getHz());

                if(!isAdmin && psc.getZhzh()&&!endwithA){
                    username+=psc.getHz();
                }
            }

            if((now - timestamp)/1000 > Long.parseLong(psc.getTimeout())){
                ResponseUtils.failedResponse(res,"token超时，请重新生成！");
                return ;
            }

            String redirect = Utils.getRedirectUrl(req,psc.getParamname());

            //登录
            FRUtils.login(req,res,username,redirect);
        }

        release(req,res,chain);
    }

    //放行拦截器
    private void release(HttpServletRequest req, HttpServletResponse res, FilterChain chain) {
        try{
            chain.doFilter(req,res);
        }catch (Exception e){
            FRUtils.FRLogInfo("拦截失败");
        }
    }

    /**
     * 拦截登录事件
     * @param res
     */
    private void login(HttpServletRequest req,HttpServletResponse res){
        JSONObject json = Utils.getRequestBody(req);
        String username =json.getString("username");
        String psd = json.getString("password");
        String dbpsd = FRUtils.getDBPsd(username,FRUtils.decryptFRPsd(psd));

        PluginSimpleConfig psc = PluginSimpleConfig.getInstance();
        boolean isWhite = IPWhiteUtils.inWhite(req,psc.getWhite());

        //返回信息
        JSONObject result = new JSONObject();

        if(!FRUtils.isUserExist(username)){
            result.put("errorCode","21300007");
            result.put("errorMsg","User not exist,or wrong password!");

            ResponseUtils.response(res,result);
            return ;
        }

        if(isWhite || !psc.getIsAll()){
            User user = FRUtils.getFRUserByUserName(username);
            if(!user.getPassword().equals(dbpsd)){
                result.put("errorCode","21300007");
                result.put("errorMsg","User not exist,or wrong password!");

                ResponseUtils.response(res,result);
                return ;
            }
            String token = FRUtils.getToken(req,res,username);

            result.put("username",username);
            result.put("validity",-1);
            result.put("callback","");
            result.put("accessToken",token);

            JSONObject origin = new JSONObject();
            origin.put("originUrl","/webroot/decision");
            origin.put("method","GET");
            origin.put("parameters",new JSONArray());

            result.put("originUrlResponse",origin);
            result.put("url","/webroot/decision");
            JSONObject data = new JSONObject();
            data.put("data",result);
            ResponseUtils.response(res,data);
            return ;
        }

        boolean isAdmin = false;
        try {
            isAdmin = Utils.isAdmin(username);
        } catch (Exception e) {
            result.put("errorMsg","9999");
            result.put("errorCode","判断管理员失败!");

            ResponseUtils.response(res,result);
            return ;

        }

        //管理员且管理员权限控制开启
        if(isAdmin && psc.getIsAdmin()){
            result.put("errorMsg","9999");
            result.put("errorCode","外网环境管理员禁止登陆!");

            ResponseUtils.response(res,result);
            return ;
        }

        //普通用户权限开始
        if(!isAdmin && psc.getIsPt()){
            result.put("errorMsg","9999");
            result.put("errorCode","外网环境普通账户禁止登陆!");

            ResponseUtils.response(res,result);
            return ;
        }

        User user = FRUtils.getFRUserByUserName(username);
        if(!user.getPassword().equals(dbpsd)){
            result.put("errorCode","21300007");
            result.put("errorMsg","User not exist,or wrong password!");

            ResponseUtils.response(res,result);
            return ;
        }
        String token = FRUtils.getToken(req,res,username);

        result.put("username",username);
        result.put("validity",-1);
        result.put("callback","");
        result.put("accessToken",token);

        JSONObject origin = new JSONObject();
        origin.put("originUrl","/webroot/decision");
        origin.put("method","GET");
        origin.put("parameters",new JSONArray());

        result.put("originUrlResponse",origin);
        result.put("url","/webroot/decision");

        JSONObject data = new JSONObject();
        data.put("data",result);

        ResponseUtils.response(res,data);
    }
}

